program chatserv;

{
  A server for a very simple chat system.
  
  Rules:
  ------
  
  Every received line that starts with "/" is interpreted as a command.
  There is only one command, "/NAME <name>" (the first part is case-
  insensitive). It is used to change a client's chat name. The answer
  is either "OK" or "DENIED <reason>".
  
  All other lines are interpreted as text and are broadcast to the
  other connected clients (but NOT back to the sending client).
  
  It is easy to access this chat system through Telnet. Writing a
  dedicated client program is, and I always wanted to say that, left
  to the reader as an exercise.
  
  Other features you might want to add to the server:
  
   - Nickname change notification
   - Dynamical user management (i. e. linked list)
}

uses
   sockets, inetaux, myerror, linux;

const
   ListenPort : Word = $AFFE;
   MaxConn = 5;
   MaxClients = 5;

type
   ClientRec = record
      cSock : LongInt;
      adstr : String;
      Nick : String;
      sin, sout : Text;
   end;

var
   Clients : array[1..MaxClients] of ClientRec;
   NumClients : LongInt;

function StripName(s : String) : String;
var i : Integer;
begin
   i := Pos(' ', s);
   while (i < Length(s)) and (s[i + 1] = ' ') do Inc(i);
   s := Copy(s, i + 1, Length(s) - i);
   i := Pos(' ', s);
   if i > 0 then Delete(s, i, Length(s) - i);
   StripName := s;
end;

var
   lSock, uSock : LongInt;
   sAddr : TInetSockAddr;
   Len, i, j : LongInt;
   Line, Cmd : String;
   MaxFD : LongInt;
   ReadSet : FDSet;
   HaveName : Boolean;

begin
   lSock := Socket(af_inet, sock_stream, 0);
   if lSock = -1 then SockError('Socket: ');
   
   with sAddr do
   begin
      Family := af_inet;
      Port := htons(ListenPort);
      Addr := 0;
   end;
   
   if not Bind(lSock, sAddr, sizeof(sAddr)) then SockError('Bind: ');
   if not Listen(lSock, MaxConn) then SockError('Listen: ');
   
   Say('Waiting...');
   
   fd_zero(ReadSet);
   NumClients := 0;
   
   repeat
      MaxFD := 0;
      for i := 1 to NumClients do with Clients[i] do
      begin
         fd_set(cSock, ReadSet);
         if cSock > MaxFD then MaxFD := cSock;
      end;
      fd_set(lSock, ReadSet);
      if lSock > MaxFD then MaxFD := lSock;
      Inc(MaxFD);
      
      Select(MaxFD, @ReadSet, nil, nil, nil);  { No timeout! }
      
      { New connections? }
      
      if fd_isset(lSock, ReadSet) then
      begin
         Say('Incoming connection.');
         if NumClients < MaxClients then  { Still room for new client? }
         begin
            Len := sizeof(sAddr);
            uSock := Accept(lSock, sAddr, Len);
            if uSock = -1 then SockSay('Accept: ')
            else
            begin
               Inc(NumClients);
               with Clients[NumClients] do
               begin
                  cSock := uSock;
                  Sock2Text(cSock, sin, sout);
                  Reset(sin);
                  Rewrite(sout);
                  adstr := AddrToStr(sAddr.Addr);
                  Nick := '';
                  Say('Accepted connection from ' + adstr);
               end;
            end;
         end;
      end;
      
      { And/or an event on an existing connection? }
      
      for i := 1 to NumClients do if fd_isset(Clients[i].cSock, ReadSet) then
      begin
         if eof(Clients[i].sin) then  { Connection has been closed? }
         begin
            with Clients[i] do
            begin
               Close(sin);
               Close(sout);
               Shutdown(cSock, 2);
               Say('Disconnected ' + adstr);
            end;
            for j := i to NumClients - 1 do Clients[j] := Clients[j + 1];
            Dec(NumClients);
         end
         else with Clients[i] do  { No -> Data can be read }
         begin
            Readln(sin, Line);
            if Pos('/', Line) = 1 then  { Command }
            begin
               Cmd := Copy(Line, 2, Pos(' ', Line) - 2);
               for j := 1 to Length(Cmd) do Cmd[j] := UpCase(Cmd[j]);
               if Cmd = 'NAME' then
               begin
                  Line := StripName(Line);
                  if Line = '' then Writeln(sout, 'DENIED no_name')
                  else begin
                     HaveName := False;
                     for j := 1 to NumClients do if Clients[j].Nick = Line
                     then HaveName := True;
                     if HaveName then Writeln(sout, 'DENIED name_in_use')
                     else Nick := Line;
                  end;
               end
               else Writeln(sout, 'UNSUPPORTED COMMAND');
            end
            else begin
               if Nick = '' then Writeln(sout, 'DENIED no_name')
               else for j := 1 to NumClients do if i <> j then
                  Writeln(Clients[j].sout, Nick, ': ', Line);
            end;
         end;
      end;
   until False;
end.
